@@ -1,3 +1,4 @@
+import asyncio
 import re
 from contextlib import asynccontextmanager
 from functools import singledispatch
@@ -56,9 +57,12 @@ async def confidence(
     dpi: int = AIOPYTESSERACT_DEFAULT_DPI,
     lang: str = AIOPYTESSERACT_DEFAULT_LANGUAGE,
     oem: int = AIOPYTESSERACT_DEFAULT_OEM,
+    timeout: float = AIOPYTESSERACT_DEFAULT_TIMEOUT,
 ) -> Optional[str]:
     proc = await execute_cmd(f"stdin stdout -l {lang} --dpi {dpi} --psm 0 --oem {oem}")
-    stdout, stderr = await proc.communicate(Path(image).read_bytes())
+    stdout, stderr = await asyncio.wait_for(
+        proc.communicate(Path(image).read_bytes()), timeout=timeout
+    )
     m = re.search(
         r"(Script.confidence:.(\d{1,10}.\d{1,10})$)",
         stdout.decode(AIOPYTESSERACT_DEFAULT_ENCODING),
@@ -99,7 +103,7 @@ async def image_to_string(
     psm: int = AIOPYTESSERACT_DEFAULT_PSM,
     oem: int = AIOPYTESSERACT_DEFAULT_OEM,
     timeout: float = AIOPYTESSERACT_DEFAULT_TIMEOUT,
-):
+) -> str:
     """Extract string from an image.
 
     :param image: image input to tesseract. (valid values: str, bytes)
@@ -315,7 +319,9 @@ async def _(
 
 
 @singledispatch
-async def image_to_boxes(image: Any) -> str:
+async def image_to_boxes(
+    image: Any, timeout: float = AIOPYTESSERACT_DEFAULT_TIMEOUT
+) -> str:
     """Bounding box estimates.
 
     :param image: image input to tesseract. (valid values: str, bytes)
@@ -324,22 +330,22 @@ async def image_to_boxes(image: Any) -> str:
 
 
 @image_to_boxes.register(str)
-async def _(image: str) -> str:
-    boxes = await image_to_boxes(Path(image).read_bytes())
+async def _(image: str, timeout: float = AIOPYTESSERACT_DEFAULT_TIMEOUT) -> str:
+    boxes = await image_to_boxes(Path(image).read_bytes(), timeout)
     return boxes
 
 
 @image_to_boxes.register(bytes)
-async def _(image: bytes) -> str:
+async def _(image: bytes, timeout: float = AIOPYTESSERACT_DEFAULT_TIMEOUT) -> str:
     proc = await execute_cmd("stdin stdout batch.nochop makebox")
-    stdout, stderr = await proc.communicate(image)
+    stdout, stderr = await asyncio.wait_for(proc.communicate(image), timeout=timeout)
     if proc.returncode != 0:
         raise TesseractRuntimeError(stderr.decode(AIOPYTESSERACT_DEFAULT_ENCODING))
     return stdout.decode(AIOPYTESSERACT_DEFAULT_ENCODING)  # type: ignore
 
 
 @singledispatch
-async def image_to_data(image: Any, dpi: float = AIOPYTESSERACT_DEFAULT_DPI) -> str:
+async def image_to_data(image: Any, dpi: int = AIOPYTESSERACT_DEFAULT_DPI) -> str:
     """Information about boxes, confidences, line and page numbers.
 
     :param image: image input to tesseract. (valid values: str, bytes)
@@ -349,15 +355,19 @@ async def image_to_data(image: Any, dpi: float = AIOPYTESSERACT_DEFAULT_DPI) ->
 
 
 @image_to_data.register(str)
-async def _(image: str, dpi: float = AIOPYTESSERACT_DEFAULT_DPI) -> str:
+async def _(image: str, dpi: int = AIOPYTESSERACT_DEFAULT_DPI) -> str:
     resp = await image_to_data(Path(image).read_bytes(), dpi)
     return resp
 
 
 @image_to_data.register(bytes)
-async def _(image: bytes, dpi: float = AIOPYTESSERACT_DEFAULT_DPI) -> str:
+async def _(
+    image: bytes,
+    dpi: int = AIOPYTESSERACT_DEFAULT_DPI,
+    timeout: float = AIOPYTESSERACT_DEFAULT_TIMEOUT,
+) -> str:
     proc = await execute_cmd(f"stdin stdout -c tessedit_create_tsv=1 --dpi {dpi}")
-    stdout, stderr = await proc.communicate(image)
+    stdout, stderr = await asyncio.wait_for(proc.communicate(image), timeout=timeout)
     if proc.returncode != 0:
         raise TesseractRuntimeError(stderr.decode(AIOPYTESSERACT_DEFAULT_ENCODING))
     return stdout.decode(AIOPYTESSERACT_DEFAULT_ENCODING)  # type: ignore
@@ -369,7 +379,7 @@ async def image_to_osd(
     dpi: int = AIOPYTESSERACT_DEFAULT_DPI,
     oem: int = AIOPYTESSERACT_DEFAULT_OEM,
     timeout: float = AIOPYTESSERACT_DEFAULT_TIMEOUT,
-):
+) -> str:
     """Information about orientation and script detection.
 
     :param image: image input to tesseract. (valid values: str, bytes)
@@ -386,7 +396,7 @@ async def _(
     dpi: int = AIOPYTESSERACT_DEFAULT_DPI,
     oem: int = AIOPYTESSERACT_DEFAULT_OEM,
     timeout: float = AIOPYTESSERACT_DEFAULT_TIMEOUT,
-):
+) -> str:
     resp = await execute(image, FileFormat.OSD, dpi, None, 0, oem, timeout)
     return resp.decode(AIOPYTESSERACT_DEFAULT_ENCODING)
 
@@ -397,7 +407,7 @@ async def _(
     dpi: int = AIOPYTESSERACT_DEFAULT_DPI,
     oem: int = AIOPYTESSERACT_DEFAULT_OEM,
     timeout: float = AIOPYTESSERACT_DEFAULT_TIMEOUT,
-):
+) -> str:
     resp = await execute(image, FileFormat.OSD, dpi, None, 0, oem, timeout)
     return resp.decode(AIOPYTESSERACT_DEFAULT_ENCODING)
 
